/**
 * Copyright (C) 2010-2012 Agrosense info@agrosense.eu
 *
 * Licensed under the Eclipse Public License - v 1.0 (the "License"); you may
 * not use this file except in compliance with the License. You may obtain a
 * copy of the License at
 *
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */
package nl.cloudfarming.client.stock.project;

import java.awt.Image;
import java.awt.datatransfer.Transferable;
import java.awt.event.ActionEvent;
import java.io.Serializable;
import javax.swing.AbstractAction;
import javax.swing.Action;
import org.netbeans.core.api.multiview.MultiViews;
import org.netbeans.spi.project.ui.LogicalViewProvider;
import org.netbeans.spi.project.ui.support.CommonProjectActions;
import org.openide.filesystems.FileObject;
import org.openide.loaders.DataFolder;
import org.openide.loaders.DataObject;
import org.openide.loaders.DataObjectNotFoundException;
import org.openide.nodes.AbstractNode;
import org.openide.nodes.Children;
import org.openide.nodes.FilterNode;
import org.openide.nodes.Node;
import org.openide.util.*;
import org.openide.util.datatransfer.PasteType;
import org.openide.util.lookup.Lookups;
import org.openide.util.lookup.ProxyLookup;
import org.openide.windows.TopComponent;

/**
 * LogicalView which wraps a the stock projectNode in a new node
 *
 * @author Merijn Zengers
 */
public class StockProjectLogicalView implements LogicalViewProvider {

    private final StockProject project;

    StockProjectLogicalView(StockProject project) {
        this.project = project;
    }

    @Override
    public Node createLogicalView() {
        try {
            //Get the Text directory, creating if deleted
            FileObject projectDirectory = project.getProjectDirectory();
//            DataObject stockObject = DataObject.find(projectDirectory.getFileObject(StockProjectFactory.STOCK_FILE));
            DataFolder projectFolder = DataFolder.findFolder(projectDirectory);

            //Get its default node-we'll wrap our node around it to change the
            //display name, icon, etc
            Node projectNode = projectFolder.getNodeDelegate();

            //This FilterNode will be our project node
            return new StockNode(projectNode, project);

        } catch (DataObjectNotFoundException donfe) {
            Exceptions.printStackTrace(donfe);
            //Fallback-the directory couldn't be created -
            //read-only filesystem or something evil happened
            return new AbstractNode(Children.LEAF);
        }
    }

    /**
     * This is the node you actually see in the project tab for the project
     */
    private static final class StockNode extends FilterNode implements Serializable{

        final StockProject project;

        public StockNode(Node node, StockProject project) throws DataObjectNotFoundException {
            super(node, new FilterNode.Children(node),
                    new ProxyLookup(new Lookup[]{Lookups.singleton(project),
                        node.getLookup()
                    }));
            this.project = project;
        }

        @Override
        public Action[] getActions(boolean arg0) {
            Action[] nodeActions = new Action[6];
            nodeActions[0] = new OpenStockDetailsAction();
            nodeActions[1] = CommonProjectActions.copyProjectAction();
            nodeActions[2] = CommonProjectActions.deleteProjectAction();
            nodeActions[4] = CommonProjectActions.closeProjectAction();
            nodeActions[5] = getOriginal().getPreferredAction();
            return nodeActions;
        }

        @Override
        public Image getIcon(int type) {
            return ImageUtilities.loadImage(StockProjectFactory.ICON);
        }

        @Override
        public Image getOpenedIcon(int type) {
            return getIcon(type);
        }

        @Override
        public String getDisplayName() {
            return project.getProjectDirectory().getName();
        }

        @Override
        public HelpCtx getHelpCtx() {
            return new HelpCtx("nl.cloudfarming.client.stock.project.about");
        }

        @Override
        public PasteType getDropType(Transferable t, int action, int index) {
            return null;
        }

        @NbBundle.Messages(value = "Stock open details action name=Open")
        private final class OpenStockDetailsAction extends AbstractAction {

            public OpenStockDetailsAction() {
                super(Bundle.Stock_open_details_action_name());
            }

            @Override
            public void actionPerformed(ActionEvent e) {
                TopComponent tc = MultiViews.createMultiView(StockProject.MIME_TYPE, StockNode.this);
                tc.open();
                tc.requestActive();
            }
        }
    }

    @Override
    public Node findPath(Node root, Object target) {
        return null;
    }
}
